@@ -13,7 +13,6 @@
 #include "auth.h"
 #include "bt-info.h"
 #include "lockscreen.h"
-#include "media-player.h"
 
 #include <locale.h>
 #include <string.h>
@@ -486,9 +485,6 @@ phosh_lockscreen_class_init (PhoshLockscreenClass *klass)
   GObjectClass *object_class = (GObjectClass *)klass;
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  g_type_ensure (PHOSH_TYPE_BT_INFO);
-  g_type_ensure (PHOSH_TYPE_MEDIA_PLAYER);
-
   object_class->constructed = phosh_lockscreen_constructed;
   object_class->dispose = phosh_lockscreen_dispose;
 
@@ -496,7 +492,7 @@ phosh_lockscreen_class_init (PhoshLockscreenClass *klass)
       G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
       NULL, G_TYPE_NONE, 0);
   /**
-   * PhoshLockscreen::wakeup-screen
+   * PhoshLockscreen::wakeup-output
    * @self: The #PhoshLockscreen emitting this signal
    *
    * Emitted when the output showing the lock screen should be woken
@@ -560,3 +556,21 @@ phosh_lockscreen_new (gpointer layer_shell,
                        "namespace", "phosh lockscreen",
                        NULL);
 }
+
+/**
+ * phosh_lockscreen_get_page
+ * @self: The #PhoshLockscreen
+ *
+ * Returns: The #PhoshLockscreenPage that is currently shown
+ */
+PhoshLockscreenPage
+phosh_lockscreen_get_page (PhoshLockscreen *self)
+{
+  PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+  gdouble position = hdy_carousel_get_position (HDY_CAROUSEL (priv->carousel));
+
+  if (position <= 0)
+    return PHOSH_LOCKSCREEN_PAGE_DEFAULT;
+  else
+    return PHOSH_LOCKSCREEN_PAGE_UNLOCK;
+}
