@@ -14,9 +14,6 @@
 
 #include "config.h"
 
-#include "bt-info.h"
-#include "connectivity-info.h"
-#include "docked-info.h"
 #include "panel.h"
 #include "shell.h"
 #include "session-manager.h"
@@ -79,9 +76,6 @@ on_shutdown_action (GSimpleAction *action,
 
   g_return_if_fail (PHOSH_IS_PANEL (self));
   phosh_session_manager_shutdown (sm);
-  /* TODO: Since we don't implement
-   * gnome.SessionManager.EndSessionDialog yet */
-  phosh_session_manager_shutdown (sm);
   phosh_panel_fold (self);
 }
 
@@ -97,9 +91,6 @@ on_restart_action (GSimpleAction *action,
   g_return_if_fail (PHOSH_IS_PANEL (self));
   g_return_if_fail (PHOSH_IS_SESSION_MANAGER (sm));
 
-  phosh_session_manager_reboot (sm);
-  /* TODO: Since we don't implement
-   * gnome.SessionManager.EndSessionDialog yet */
   phosh_session_manager_reboot (sm);
   phosh_panel_fold (self);
 }
@@ -384,11 +375,6 @@ phosh_panel_class_init (PhoshPanelClass *klass)
       G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
       NULL, G_TYPE_NONE, 0);
 
-  g_type_ensure (PHOSH_TYPE_BT_INFO);
-  g_type_ensure (PHOSH_TYPE_CONNECTIVITY_INFO);
-  g_type_ensure (PHOSH_TYPE_DOCKED_INFO);
-  g_type_ensure (PHOSH_TYPE_SETTINGS);
-
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/sm/puri/phosh/ui/top-panel.ui");
   gtk_widget_class_bind_template_child_private (widget_class, PhoshPanel, btn_top_panel);
