@@ -51,7 +51,7 @@ static guint signals[N_SIGNALS] = { 0 };
 
 struct _PhoshPolkitAuthPrompt
 {
-  PhoshLayerSurface parent;
+  PhoshSystemModalDialog parent;
 
   GtkWidget *lbl_message;
   GtkWidget *lbl_user_name;
@@ -75,7 +75,7 @@ struct _PhoshPolkitAuthPrompt
 
   gboolean done_emitted;
 };
-G_DEFINE_TYPE(PhoshPolkitAuthPrompt, phosh_polkit_auth_prompt, PHOSH_TYPE_LAYER_SURFACE);
+G_DEFINE_TYPE(PhoshPolkitAuthPrompt, phosh_polkit_auth_prompt, PHOSH_TYPE_SYSTEM_MODAL_DIALOG);
 
 
 static void phosh_polkit_auth_prompt_initiate (PhoshPolkitAuthPrompt *self);
@@ -365,8 +365,10 @@ phosh_polkit_auth_prompt_initiate (PhoshPolkitAuthPrompt *self)
 
 
 static void
-on_btn_cancel_clicked (PhoshPolkitAuthPrompt *self, GtkButton *btn)
+on_dialog_canceled (PhoshPolkitAuthPrompt *self, gpointer unused)
 {
+  g_return_if_fail (PHOSH_IS_POLKIT_AUTH_PROMPT (self));
+
   polkit_agent_session_cancel (self->session);
   emit_done (self, TRUE);
 }
@@ -389,27 +391,6 @@ on_btn_authenticate_clicked (PhoshPolkitAuthPrompt *self, GtkButton *btn)
 }
 
 
-static gboolean
-on_key_press_event (PhoshPolkitAuthPrompt *self, GdkEventKey *event, gpointer data)
-{
-  gboolean handled = FALSE;
-  g_return_val_if_fail (PHOSH_IS_POLKIT_AUTH_PROMPT (self), FALSE);
-
-  switch (event->keyval) {
-    case GDK_KEY_Escape:
-      polkit_agent_session_cancel (self->session);
-      emit_done (self, TRUE);
-      handled = TRUE;
-      break;
-    default:
-      /* nothing to do */
-      break;
-  }
-
-  return handled;
-}
-
-
 static void
 phosh_polkit_auth_prompt_dispose (GObject *obj)
 {
@@ -450,7 +431,7 @@ phosh_polkit_auth_prompt_constructed (GObject *object)
 
   g_signal_connect_object (self->btn_cancel,
                            "clicked",
-                           G_CALLBACK (on_btn_cancel_clicked),
+                           G_CALLBACK (on_dialog_canceled),
                            self,
                            G_CONNECT_SWAPPED);
   g_signal_connect_object (self->btn_authenticate,
@@ -459,24 +440,6 @@ phosh_polkit_auth_prompt_constructed (GObject *object)
                            self,
                            G_CONNECT_SWAPPED);
 
-  gtk_widget_add_events (GTK_WIDGET (self), GDK_KEY_PRESS_MASK);
-  g_signal_connect (G_OBJECT (self),
-                    "key_press_event",
-                    G_CALLBACK (on_key_press_event),
-                    NULL);
-
-  {
-    GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
-    GtkWidget *lbl = gtk_label_new (_("Authenticate"));
-
-    self->spinner_authenticate = gtk_spinner_new ();
-    gtk_widget_set_no_show_all (self->spinner_authenticate, TRUE);
-    gtk_container_add (GTK_CONTAINER (self->btn_authenticate), box);
-    gtk_container_add (GTK_CONTAINER (box), self->spinner_authenticate);
-    gtk_container_add (GTK_CONTAINER (box), lbl);
-    gtk_widget_show_all (self->btn_authenticate);
-  }
-
   phosh_polkit_auth_prompt_initiate (self);
 }
 
@@ -546,6 +509,8 @@ phosh_polkit_auth_prompt_class_init (PhoshPolkitAuthPromptClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, btn_authenticate);
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, btn_cancel);
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, entry_password);
+  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, spinner_authenticate);
+  gtk_widget_class_bind_template_callback (widget_class, on_dialog_canceled);
 }
 
 
@@ -561,9 +526,7 @@ phosh_polkit_auth_prompt_new (const char *action_id,
                               const char *message,
                               const char *icon_name,
                               const char *cookie,
-                              GStrv user_names,
-                              gpointer layer_shell,
-                              gpointer wl_output)
+                              GStrv user_names)
 {
   return g_object_new (PHOSH_TYPE_POLKIT_AUTH_PROMPT,
                        /* polkit prompt */
@@ -572,16 +535,5 @@ phosh_polkit_auth_prompt_new (const char *action_id,
                        "message", message,
                        "icon-name", icon_name,
                        "user-names", user_names,
-                       /* layer shell */
-                       "layer-shell", layer_shell,
-                       "wl-output", wl_output,
-                       "anchor", ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP |
-                       ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM |
-                       ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT |
-                       ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT,
-                       "layer", ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
-                       "kbd-interactivity", TRUE,
-                       "exclusive-zone", -1,
-                       "namespace", "phosh prompter",
                        NULL);
 }
