@@ -10,22 +10,17 @@
 
 #include <glib/gi18n.h>
 
-#include "bt-info.h"
-#include "docked-info.h"
 #include "mode-manager.h"
 #include "shell.h"
 #include "settings.h"
 #include "quick-setting.h"
 #include "settings/brightness.h"
 #include "settings/gvc-channel-bar.h"
-#include "torch-info.h"
 #include "wwan/phosh-wwan-mm.h"
-#include "rotateinfo.h"
-#include "feedbackinfo.h"
 #include "feedback-manager.h"
 #include "notifications/notify-manager.h"
 #include "notifications/notification-frame.h"
-#include "media-player.h"
+#include "rotateinfo.h"
 
 #include <pulse/pulseaudio.h>
 #include "gvc-mixer-control.h"
@@ -88,14 +83,56 @@ static void
 rotation_setting_clicked_cb (PhoshSettings *self)
 {
   PhoshShell *shell = phosh_shell_get_default ();
+  PhoshRotationManager *rotation_manager;
+  PhoshRotationManagerMode mode;
   PhoshMonitorTransform transform;
+  gboolean locked;
 
   g_return_if_fail (PHOSH_IS_SETTINGS (self));
-  transform = phosh_shell_get_transform (shell);
-  phosh_shell_set_transform (shell, transform == PHOSH_MONITOR_TRANSFORM_NORMAL
-                             ? PHOSH_MONITOR_TRANSFORM_270
-                             : PHOSH_MONITOR_TRANSFORM_NORMAL);
-  g_signal_emit (self, signals[SETTING_DONE], 0);
+
+  rotation_manager = phosh_shell_get_rotation_manager (shell);
+  g_return_if_fail (rotation_manager);
+  mode = phosh_rotation_manager_get_mode (PHOSH_ROTATION_MANAGER (rotation_manager));
+
+  switch (mode) {
+  case PHOSH_ROTATION_MANAGER_MODE_OFF:
+    transform = phosh_rotation_manager_get_transform (rotation_manager) ?
+      PHOSH_MONITOR_TRANSFORM_NORMAL : PHOSH_MONITOR_TRANSFORM_270;
+    phosh_rotation_manager_set_transform (rotation_manager, transform);
+    g_signal_emit (self, signals[SETTING_DONE], 0);
+    break;
+  case PHOSH_ROTATION_MANAGER_MODE_SENSOR:
+    locked = phosh_rotation_manager_get_orientation_locked (rotation_manager);
+    phosh_rotation_manager_set_orientation_locked (rotation_manager, !locked);
+    break;
+  default:
+    g_assert_not_reached ();
+  }
+}
+
+static void
+rotation_setting_long_pressed_cb (PhoshSettings *self)
+{
+  PhoshShell *shell = phosh_shell_get_default ();
+  PhoshRotateInfoMode mode;
+  PhoshRotationManager *rotation_manager;
+
+  rotation_manager = phosh_shell_get_rotation_manager (shell);
+  g_return_if_fail (rotation_manager);
+
+  mode = phosh_rotation_manager_get_mode (rotation_manager);
+  switch (mode) {
+  case PHOSH_ROTATION_MANAGER_MODE_OFF:
+    mode = PHOSH_ROTATION_MANAGER_MODE_SENSOR;
+    break;
+  case PHOSH_ROTATION_MANAGER_MODE_SENSOR:
+    mode = PHOSH_ROTATION_MANAGER_MODE_OFF;
+    break;
+  default:
+    g_assert_not_reached ();
+  }
+  g_debug ("Rotation manager mode: %d", mode);
+  phosh_rotation_manager_set_mode (rotation_manager, mode);
 }
 
 static void
@@ -178,6 +215,13 @@ docked_setting_clicked_cb (PhoshSettings *self)
   phosh_docked_manager_set_enabled (manager, !enabled);
 }
 
+static void
+docked_setting_long_pressed_cb (PhoshSettings *self)
+{
+  phosh_quick_setting_open_settings_panel ("display");
+  g_signal_emit (self, signals[SETTING_DONE], 0);
+}
+
 
 static void
 update_output_vol_bar (PhoshSettings *self)
@@ -229,18 +273,20 @@ mixer_control_output_update_cb (GvcMixerControl *mixer, guint id, gpointer *data
   if (self->output_stream)
     g_signal_handlers_disconnect_by_data (self->output_stream, self);
 
-  self->output_stream = gvc_mixer_control_get_default_sink (self->mixer_control);
+  g_set_object (&self->output_stream,
+                gvc_mixer_control_get_default_sink (self->mixer_control));
   g_return_if_fail (self->output_stream);
 
-  g_signal_connect (self->output_stream,
-                    "notify::volume",
-                    G_CALLBACK (output_stream_notify_volume_cb),
-                    self);
+  g_signal_connect_object (self->output_stream,
+                           "notify::volume",
+                           G_CALLBACK (output_stream_notify_volume_cb),
+                           self, 0);
+
+  g_signal_connect_object (self->output_stream,
+                           "notify::is-muted",
+                           G_CALLBACK (output_stream_notify_is_muted_cb),
+                           self, 0);
 
-  g_signal_connect (self->output_stream,
-                    "notify::is-muted",
-                    G_CALLBACK (output_stream_notify_is_muted_cb),
-                    self);
   update_output_vol_bar (self);
 }
 
@@ -253,7 +299,7 @@ vol_adjustment_value_changed_cb (GtkAdjustment *adjustment,
   g_autofree char *name = NULL;
 
   if (!self->output_stream)
-    self->output_stream = gvc_mixer_control_get_default_sink (self->mixer_control);
+    self->output_stream = g_object_ref (gvc_mixer_control_get_default_sink (self->mixer_control));
 
   volume = gtk_adjustment_get_value (adjustment);
   rounded = round (volume);
@@ -424,6 +470,8 @@ phosh_settings_dispose (GObject *object)
 
   brightness_dispose ();
 
+  g_clear_object (&self->output_stream);
+
   if (self->notify_event) {
     end_notify_feedback (self);
     g_clear_object (&self->notify_event);
@@ -462,14 +510,6 @@ phosh_settings_class_init (PhoshSettingsClass *klass)
       G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
       NULL, G_TYPE_NONE, 0);
 
-  g_type_ensure (PHOSH_TYPE_BT_INFO);
-  g_type_ensure (PHOSH_TYPE_DOCKED_INFO);
-  g_type_ensure (PHOSH_TYPE_FEEDBACK_INFO);
-  g_type_ensure (PHOSH_TYPE_MEDIA_PLAYER);
-  g_type_ensure (PHOSH_TYPE_QUICK_SETTING);
-  g_type_ensure (PHOSH_TYPE_ROTATE_INFO);
-  g_type_ensure (PHOSH_TYPE_TORCH_INFO);
-
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, box_settings);
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, list_notifications);
   gtk_widget_class_bind_template_child (widget_class, PhoshSettings, quick_settings);
@@ -479,10 +519,12 @@ phosh_settings_class_init (PhoshSettingsClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, battery_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, bt_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, docked_setting_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, docked_setting_long_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, feedback_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, feedback_setting_long_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_media_player_raised);
   gtk_widget_class_bind_template_callback (widget_class, rotation_setting_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, rotation_setting_long_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, torch_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, wifi_setting_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, wwan_setting_clicked_cb);
