@@ -64,6 +64,14 @@ phosh_wwan_default_init (PhoshWWanInterface *iface)
                           FALSE,
                           G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY));
 
+  g_object_interface_install_property (
+    iface,
+    g_param_spec_boolean ("enabled",
+                          "Modem enabled",
+                          "Whether there modem is enabled",
+                          FALSE,
+                          G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY));
+
   g_object_interface_install_property (
     iface,
     g_param_spec_string ("operator",
@@ -140,6 +148,19 @@ phosh_wwan_is_present (PhoshWWan *self)
 }
 
 
+gboolean
+phosh_wwan_is_enabled (PhoshWWan *self)
+{
+  PhoshWWanInterface *iface;
+
+  g_return_val_if_fail (PHOSH_IS_WWAN (self), FALSE);
+
+  iface = PHOSH_WWAN_GET_IFACE (self);
+  g_return_val_if_fail (iface->is_enabled != NULL, FALSE);
+  return iface->is_enabled (self);
+}
+
+
 const char *
 phosh_wwan_get_operator (PhoshWWan *self)
 {
